classdef TREE6 < PROBLEM
% <problem> <TREE>
% The time-varying ratio error estimation problem
% T --- 1000 --- Length of data (related to the number of variables)

%------------------------------- Reference --------------------------------
% C. He, R. Cheng, C. Zhang, Y. Tian, Q. Chen, and X. Yao, Evolutionary
% large-scale multiobjective optimization for ratio error estimation of
% voltage transformers, IEEE Transactions on Evolutionary Computation,
% 2020.
%------------------------------- Copyright --------------------------------
% Copyright (c) 2018-2019 BIMK Group. You are free to use the PlatEMO for
% research purposes. All publications which use this platform or any code
% in the platform should acknowledge the use of "PlatEMO" and reference "Ye
% Tian, Ran Cheng, Xingyi Zhang, and Yaochu Jin, PlatEMO: A MATLAB platform
% for evolutionary multi-objective optimization [educational forum], IEEE
% Computational Intelligence Magazine, 2017, 12(4): 73-87".
%--------------------------------------------------------------------------

    properties(Access = private)
        Data;   % Dataset
        Mean;   % Mean values of the dataset
    end
    methods
        %% Initialization
        function obj = TREE6()
            % Load data
            T = obj.Global.ParameterSet(1000);
            CallStack = dbstack('-completenames');
            load(fullfile(fileparts(CallStack(1).file),'Dataset_TREE.mat'),'Dataset');
            obj.Data = Dataset.TREE6(1:min(max(T,10),end),:);
            % Set the numbers of objectives and decision variables
            K = 12;
            obj.Global.M = 3;
            obj.Global.D = T*K;
            % Set the upper and lower boundaries
            Lower    = zeros(T,K);
            Upper    = zeros(T,K);
            obj.Mean = zeros(T,K);
            for i = 1 : 3
                Lower(:,i)      = min(obj.Data(:,i:3:end*0.25),[],2)/1.01;
                Upper(:,i)      = max(obj.Data(:,i:3:end*0.25),[],2)/0.99;
                Lower(:,i+3)    = min(obj.Data(:,end*0.25+i:3:end*0.5),[],2)/1.01;
                Upper(:,i+3)    = max(obj.Data(:,end*0.25+i:3:end*0.5),[],2)/0.99;
                Lower(:,i+6)    = min(obj.Data(:,end*0.5+i:3:end*0.75),[],2) - pi/180;
                Upper(:,i+6)    = max(obj.Data(:,end*0.5+i:3:end*0.75),[],2) + pi/180;
                Lower(:,i+9)    = min(obj.Data(:,end*0.75+i:3:end),[],2) - pi/180;
                Upper(:,i+9)    = max(obj.Data(:,end*0.75+i:3:end),[],2) + pi/180;
                obj.Mean(:,i)   = mean(obj.Data(:,i:3:end*0.25),2);
                obj.Mean(:,i+3) = mean(obj.Data(:,end*0.25+i:3:end*0.5),2);
                obj.Mean(:,i+6) = mean(obj.Data(:,end*0.5+i:3:end*0.75),2);
                obj.Mean(:,i+9) = mean(obj.Data(:,end*0.75+i:3:end),2);
            end
            % The decision variables are the offset of the mean values of
            % the dataset
            obj.Global.lower    = reshape(Lower-obj.Mean,1,[]);
            obj.Global.upper    = reshape(Upper-obj.Mean,1,[]);
            obj.Global.encoding = 'real';
        end
        %% Generate initial population
        function PopDec = Init(obj,N)
            PopDec = obj.Mean(:)'.*(rand(N,obj.Global.D)*0.008-0.004);
            PopDec = min(max(PopDec,repmat(obj.Global.lower,N,1)),repmat(obj.Global.upper,N,1));
        end
        %% Calculate objective values
        function PopObj = CalObj(obj,PopDec)
            N      = size(PopDec,1);
            KP     = size(obj.Data,2);
            [T,K]  = size(obj.Mean);
            PopDec = reshape(PopDec,N,T,K) + repmat(reshape(obj.Mean,1,T,K),N,1,1);
            PopObj = zeros(N,obj.Global.M);
            % First objective
            eA1 = abs(repmat(reshape(obj.Data(:,1:end*0.25),1,T,KP/4),N,1,1)./repmat(PopDec(:,:,1:3),1,1,KP/12)-1);
            eA2 = abs(repmat(reshape(obj.Data(:,end*0.25+1:end*0.5),1,T,KP/4),N,1,1)./repmat(PopDec(:,:,4:6),1,1,KP/12)-1);
            eA3 = abs(repmat(reshape(obj.Data(:,end*0.5+1:end*0.75),1,T,KP/4),N,1,1)./repmat(PopDec(:,:,7:9),1,1,KP/12)-1);
            eA4 = abs(repmat(reshape(obj.Data(:,end*0.75+1:end),1,T,KP/4),N,1,1)./repmat(PopDec(:,:,10:12),1,1,KP/12)-1);
            eA  = cat(3,eA1,eA2);
            eAA = cat(3,eA3,eA4);
            PopObj(:,1) = sum(sum(eA,3),2);
            % Second objective
            Delta = std(eA(:,2:end,:)-eA(:,1:end-1,:),0,2);
            PopObj(:,2) = sum(reshape(Delta,N,[]),2);
            % Third objective
            Delta = std(eAA(:,2:end,:)-eAA(:,1:end-1,:),0,2);
            PopObj(:,3) = sum(reshape(Delta,N,[]),2);
        end
        %% Calculate constraint violations
        function PopCon = CalCon(obj,PopDec)
            N      = size(PopDec,1);
            [T,K]  = size(obj.Mean);
            PopDec = reshape(PopDec,N,T,K) + repmat(reshape(obj.Mean,1,T,K),N,1,1);
            PopCon = TREE_CalCon(PopDec,3);
        end
        %% A reference point for hypervolume calculation
        function P = PF(obj,N)
            X = zeros(1,obj.Global.D);
            X(1:2:end) = obj.Global.lower(1:2:end);
            X(2:2:end) = obj.Global.upper(2:2:end);
            P = obj.CalObj(X);
        end
    end
end